
function coerceClass(className) {
	//Return class for className, or null if not found
	if (typeof className === 'function')
		return className;
	let cl = null;
	try {
		const fct = new Function(`return ${className}`);	
		cl = fct(className);
		} catch(ex) {
		//not found, nop
	}
	return cl;
}


function isStringValidNumber(str) {
	if(str.match(/^-?\d+$/))
		return true;
	if (str.match(/^\d+\.\d+$/))
		return true;
	return false;
}

function isString(o) {
	return typeof o === 'string';
}

//-----------------------------------------------------------
//Coercing

function coerceNum(o) {
	if (!isString(o))
		return o;
	//it is a string
	if (!isStringValidNumber(o))
		throw new CustomException(`Invalid number "${o}"`);
	return parseFloat(o);
}
	
function coerceNumWithDefault(o, defaultNum) {
	//if number is not valid use defaultNum
	if (!o) { return defaultNum; }
	if (isString(o) && !isStringValidNumber(o))
		return defaultNum;
	return coerceNum(o);
}

//-----------------------------------------------------------
//Validating

//function parseNumsString(anInput) {
function coerceNumsString(anInput) {	
	//Assume pre-validated
	//Allow either whitespace or comma delimiter
	if (!anInput)
		return [];
	const input = anInput.trim();
	//Handle special case first
	if (input.length === 0)
		return [];
	const strings = input.trim().split(/[\s,]+/);
	return strings.map(eaString => parseFloat(eaString));
}

function findInvalidNumberIn(numsString) {
	//validate a numbers string
	//Return null if okay
	//If invalid number (format) is found, return it
	const s = numsString.trim();
	if (s.length === 0) return null;
	const strings = s.split(/[\s,]+/);	
	for (let each of strings)
		if (!isStringValidNumber(each))
			return each;
	return null;
}

//-----------------------------------------------------------
//Helpers

function className(o) {
	return o.constructor.name;
}

function implementedBySubclass(o, methodName) {
	throw new CustomException(`Class "${className(o)}" must implement method "${methodName}"`);
}

function decimalCount(num) {
	if (Math.floor(num) === num)
		return 0;
	const s = num.toString();
	if (!s.includes('.'))
		return 0;
	return s.split('.')[1].length;
}

//-----------------------------------------------------------
//Exception Helpers

function exceptionToDisplayStrings(ex) {
	const strings = [];
	try {
		const lines = ex.stack.split('\n');
		//Assume 0 is info, 1 is location
		strings.push(lines[0]);
		const line2 = lines[1];
		const
			messageLoc = this.parseExceptionMessageLocation(line2),
			fileLoc = this.parseExceptionFileLocation(line2);
		strings.push(`(method=${messageLoc}) ${fileLoc}`);
	} catch (ex) {
		//nop
	}
	return strings;	
}

function parseExceptionMessageLocation(stackLine) {
	//Expecting something like:
	//	"    at Counter.getCount (file:/
	let key, i, sub;
	key = 'at ';
	i = stackLine.indexOf(key);
	if (i == -1) return '';
	sub = stackLine.substring(i + key.length);
	key = ' ';
	i = sub.indexOf(key);
	if (i == -1) return '';		
	return sub.substring(0, i);
}

function parseExceptionFileLocation(stackLine) {
	//Expecting something like (at end of string):
	//	/model.js:17:5)"
	let i, sub, fn, lineNo;
	i = stackLine.lastIndexOf('/')+1;
	if (i == -1) return '';						
	sub = stackLine.substring(i);
	fn = sub.substring(0, sub.indexOf(':'));
	lineNo = sub.substring(sub.indexOf(':')+1);
	lineNo = lineNo.substring(0, lineNo.indexOf(':'));
	//foo.js (line 101)
	return `(file="${fn}") (line=${lineNo})`;
}

//============================================================
//Base system extensions

//We minimize these extensions (could collide with other libraries
//However in obvious cases (capitalize is capitalize) we deem it okay

String.prototype.capitalized = function() {
	//Note -- charAt is "soft" i.e. will return empty string for bad index
	return this.charAt(0).toUpperCase() + this.slice(1);
}


//============================================================
//Helper Classes

class CustomException extends Error {
	constructor(msg) {
		super(msg);
		this.name = "CustomException";
	}
}



