/*
	counter-controller.js
	Controller is interface and event manager between View and model
*/

class Controller extends AbstractController {
	
	//----------------------------
	//Virtual - required
	
	newModel() {
		//Override
		return new Rectangle();
	}
	
	//----------------------------
	//Virtual - optional
	
	getRefreshableFields() {
		return ['width', 'height', 'area', 'perimeter', 'diagonal'];
	}		
	
	getChangeableFields() {
		//fields that can be changed
		return ['width', 'height'];
	}	
	
	//----------------------------
	//Attach (connect) element events to handler methods
	
	attachEvents() {
		//Attach field event to event handler
		//ui element name, event name, event handler method name
		super.attachEvents();
		
		//actions
		this.attachAction('flip', 'click', 'clickedFlip');				
		this.attachAction('multiply', 'click', 'clickedMultiply');
	}
		
	//-------------------------------
	//Action event handlers -- actions (e.g. button clicks 'clicked')
	
	clickedFlip() {
		this.model.flip();
	}	
	
	clickedMultiply() {
		//Get factor from user
		const factorString = prompt("Please enter factor to multiply rectangle dimensions by", "2");	

		//If canceled, return
		if (factorString == null)
			return;
	
		//convert number to string using helper "coerceNum"
		const factor = coerceNum(factorString);
		this.model.multiply(factor);		
	}		
	
}

//======================================================
//No changes required

function buildController() {
	//one and only controller (global)
	controller = new Controller();
}	


